//*****************************************************************************
//																	2024.09
// 15puzzle Optimal Solver	WindowsUI	by takaken
//								for Microsoft Visual Studio Community 2022 
//*****************************************************************************
#include <windows.h>
#include <process.h>
#include <omp.h>
#include <time.h>
#include "resource.h"

// \o[
#include "15puz20solver.c"

const char* szTitle = "15puzzle Optimal Solver Ver2.0 (2024) by takaken";

#define  UM_PDB1_END   32766
#define  UM_PDB2_END   32767
#define  UM_WD_END	   32768
#define  UM_SOLVE_END  32769
#define  ID_MYTIMER    32770

// Status (ԊǗR[h)
#define  NORMAL 0
#define  BACK	1	// obN[h
#define  DRAG	2	// hbOҏW
#define  SOLVE	3	// T
#define  SHOW	4	// ʕ\
#define  PLAY	5	// g[X\

int  Status;		// ԊǗR[h
int  TraceData; 	// g[Xf[^̗L

HINSTANCE hInst;
HWND	  hDlog;
HFONT	  hFont;

HDC 	  BuffDC;
HDC 	  ItemDC;
HBITMAP   BuffBM;
HBITMAP   ItemBM;
RECT	  Picture;

int  PICBOX[BOARD_SIZE];	// sN`[{bNXf[^
char EditBox[500];			// GfBbg{bNXf[^

int  MOUSE_X;				// }EXW X
int  MOUSE_Y;				// }EXW Y
int  DRAG_X;				// }EXhbON_X
int  DRAG_Y;				// }EXhbON_Y

int  BackCount; 			// tsړ
int  MovePieceNum;			// ړ̃s[Xԍ
int  MovePiecePos;			// ړs[X̌ʒu
int  TraceIndex;			// TRACE[]index
int  TraceVect; 			// Tracëړ
int  TraceDiff = 8; 		// Tracëړψ (1,2,4,8,16,,)

int  NOW_TIME;				// oߎ(sec)
int  PlayStop;				// g[XXgbv

int  PDB1_END = 0;
int  PDB2_END = 0;
int  WD_END = 0;

//*****************************************************************************
// LA
//*****************************************************************************
void Enable(HWND hWnd, int status)
{
	Status = status;

	if (Status == SOLVE || Status == PLAY) {	// T or g[XĐ
		EnableWindow(GetDlgItem(hWnd, IDC_SHUFFLE), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_BACK), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_DRAG), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_RESET), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_SOLVE), FALSE);
		EnableWindow(GetDlgItem(hWnd, IDC_CANCEL), TRUE);
		EnableWindow(GetDlgItem(hWnd, IDC_TRACE), FALSE);
	}
	else {	// Tg[XĐȊO
		EnableWindow(GetDlgItem(hWnd, IDC_SHUFFLE), TRUE);
		EnableWindow(GetDlgItem(hWnd, IDC_BACK), TRUE);
		EnableWindow(GetDlgItem(hWnd, IDC_DRAG), TRUE);
		EnableWindow(GetDlgItem(hWnd, IDC_RESET), TRUE);
		EnableWindow(GetDlgItem(hWnd, IDC_SOLVE), TRUE);
		EnableWindow(GetDlgItem(hWnd, IDC_CANCEL), FALSE);
		if (Status != SHOW && Status != PLAY) {
			TraceData = FALSE;
			SetWindowText(GetDlgItem(hWnd, IDC_STATUS1), "");
			SetWindowText(GetDlgItem(hWnd, IDC_STATUS2), "0 sec");
		}
		EnableWindow(GetDlgItem(hWnd, IDC_TRACE), TraceData && MOVES > 0);
	}
}
//*****************************************************************************
// sN`[o
//*****************************************************************************
void PictureShow(void)
{
	int  i, x1, y1, x2, y2;

	// {[hS
	for (i = 0; i < BOARD_SIZE; i++) {
		x1 = (i % 4) * 64;
		y1 = (i / 4) * 64;
		x2 = (PICBOX[i] % 4) * 64;
		y2 = (PICBOX[i] / 4) * 64;
		BitBlt(BuffDC, x1, y1, 64, 64, ItemDC, x2, y2, SRCCOPY);
	}

	// ړ̃s[X
	if (MovePieceNum) {
		x1 = MOUSE_X - 32;
		y1 = MOUSE_Y - 32;
		x2 = (MovePieceNum % 4) * 64;
		y2 = (MovePieceNum / 4) * 64;
		BitBlt(BuffDC, x1, y1, 64, 64, ItemDC, x2, y2, SRCCOPY);
	}
	InvalidateRect(hDlog, &Picture, FALSE); // ĕ`
}
//*****************************************************************************
// g[X
//*****************************************************************************
void Animation(HWND hWnd)
{
	int  i, pos = 0;
	char buff[30];

	if (PlayStop) { // [LZ]NbNꂽ
		KillTimer(hWnd, (UINT)ID_MYTIMER);
		PlayStop = FALSE;
		if (MovePieceNum) { // ړ
			PICBOX[MovePiecePos] = MovePieceNum;
			MovePieceNum = TraceVect = 0;
			wsprintf(buff, " %2d / %2d moves", TraceIndex, MOVES);
			SetWindowText(GetDlgItem(hWnd, IDC_STATUS1), buff);
			PictureShow();
		}
		Enable(hWnd, SHOW);
		return;
	}

	if (MovePieceNum == 0) {										// ړJn
		MovePieceNum = TRACE[TraceIndex++];
		for (i = 0; i < BOARD_SIZE; i++) {
			if (PICBOX[i] == MovePieceNum) pos = i; // Start
			if (PICBOX[i] == 0) MovePiecePos = i;	// Finish
		}
		PICBOX[pos] = 0;
		// ړs[X̒S
		MOUSE_X = (pos % 4) * 64 + 32;
		MOUSE_Y = (pos / 4) * 64 + 32;
		// ړ
		switch (MovePiecePos - pos) {
		case -4: TraceVect = 1; MOUSE_Y -= TraceDiff; break;	// 
		case  1: TraceVect = 2; MOUSE_X += TraceDiff; break;	// E
		case  4: TraceVect = 3; MOUSE_Y += TraceDiff; break;	// 
		case -1: TraceVect = 4; MOUSE_X -= TraceDiff; break;	// 
		}
	}
	else {
		if ((MOUSE_X - 32) % 64 == 0 && (MOUSE_Y - 32) % 64 == 0) { // ړI
			PICBOX[MovePiecePos] = MovePieceNum;
			MovePieceNum = TraceVect = 0;
			wsprintf(buff, " %2d / %2d moves", TraceIndex, MOVES);
			SetWindowText(GetDlgItem(hWnd, IDC_STATUS1), buff);
		}
		else {														// ړ
			switch (TraceVect) {
			case 1: MOUSE_Y -= TraceDiff; break;	// 
			case 2: MOUSE_X += TraceDiff; break;	// E
			case 3: MOUSE_Y += TraceDiff; break;	// 
			case 4: MOUSE_X -= TraceDiff; break;	// 
			}
		}
	}

	PictureShow();
}
//*****************************************************************************
// \o[
//*****************************************************************************
// PDB1e[u쐬
void Pdb1Thread(void*)
{
	// WDe[u
	MAX_PDB1 = MakePDB(PDB1, PATTERN1);

	// XbhIƂʒm
	SendMessage(hDlog, UM_PDB1_END, 0, 0);
	_endthread();
}
// PDB2e[u쐬
void Pdb2Thread(void*)
{
	// WDe[u
	MAX_PDB2 = MakePDB(PDB2, PATTERN2);

	// XbhIƂʒm
	SendMessage(hDlog, UM_PDB2_END, 0, 0);
	_endthread();
}
// WDe[u쐬
void WdThread(void*)
{
	// WDe[u
	MakeWDDB();

	// XbhIƂʒm
	SendMessage(hDlog, UM_WD_END, 0, 0);
	_endthread();
}
// \o[s
void SolverThread(void*)
{
	int  y, x, pos, i;
	char buff[100];
	clock_t st;
	double utime;
	int ut1, ut2;

	// TO
	for (i = 0; i < BOARD_SIZE; i++)
		BOARD[i] = PICBOX[i];

	// BOARD[]GfBbg{bNXo
	*EditBox = '\0';
	for (y = 0; y < 4; y++) {
		for (x = 0; x < 4; x++) {
			pos = y * 4 + x;
			wsprintf(buff, " %2d", BOARD[pos]);
			strcat_s(EditBox, 500, buff);
		}
		strcat_s(EditBox, 500, "\r\n");
	}
	SetWindowText(GetDlgItem(hDlog, IDC_EDIT1), EditBox);

	// TJn
	st = clock();

	//----------------------------------
	// 
	//----------------------------------
	RESULT = PreSearch();

	// ygpXbh\z
	wsprintf(buff, " WD=%d, PDB=%d, Threads=%d/%d", START_WD, START_PDB, USE_THREADS, MAX_THREADS);
	SetWindowText(GetDlgItem(hDlog, IDC_STATUS1), buff);

	// OpenMPT (ŉĂȂ̂)
	//----------------------------------
	if (RESULT == FALSE) OmpSearch();
	//----------------------------------

	// s
	utime = (double)(clock() - st) / CLOCKS_PER_SEC;
	ut1 = (INT)utime;					// _ȏ
	ut2 = (int)(utime * 100.5) % 100;	// _ȉ

	// ʕ\
	if (RESULT == CANCEL) {
		SetWindowText(GetDlgItem(hDlog, IDC_STATUS1), "Cancel");
		TraceData = FALSE;
		strcat_s(EditBox, 500, "Cancel.");
	}
	else if (RESULT == UNSOLVE) {
		SetWindowText(GetDlgItem(hDlog, IDC_STATUS1), "UnSolvable");
		TraceData = FALSE;
		strcat_s(EditBox, 500, "UnSolvable.");
	}
	else {	// SOLVED
		TraceData = (MOVES > 0) ? TRUE : FALSE;
		TraceIndex = 0;

		wsprintf(buff, "Optimal solution is %d moves.\r\n", MOVES);
		strcat_s(EditBox, 500, buff);
		wsprintf(buff, "Used time: %d.%02d sec\r\n", ut1, ut2);
		strcat_s(EditBox, 500, buff);
		strcat_s(EditBox, 500, "Solution:\r\n");
		for (i = 0; i < MOVES; i++) {
			if (i && (i % 10) == 0) strcat_s(EditBox, 500, "\r\n");
			wsprintf(buff, " %2d", TRACE[i]);
			strcat_s(EditBox, 500, buff);
		}

		// Xe[^Xo[\̎ԕ␳
		wsprintf(buff, "%5d sec", ut1);
		SetWindowText(GetDlgItem(hDlog, IDC_STATUS2), buff);
	}
	SetWindowText(GetDlgItem(hDlog, IDC_EDIT1), EditBox);

	// XbhIƂʒm
	SendMessage(hDlog, UM_SOLVE_END, 0, 0);
	_endthread();
}
//*****************************************************************************
// Zbg
//*****************************************************************************
void Reset(void)
{
	int  pos;

	for (pos = 0; pos < 15; pos++)
		PICBOX[pos] = pos + 1;
	PICBOX[15] = 0;
	PictureShow();
}
//*****************************************************************************
// _CNghbO
//*****************************************************************************
void DirctDragGet(void)
{
	int  x, y;
	RECT rc;

	x = MOUSE_X / 64;
	y = MOUSE_Y / 64;

	MovePiecePos = y * 4 + x;
	MovePieceNum = PICBOX[MovePiecePos];
	if (MovePieceNum == 0) return;

	PICBOX[MovePiecePos] = 0;
	PictureShow();

	// hbO͈͂߂
	GetWindowRect(hDlog, &rc);
	rc.left += DRAG_X;
	rc.top += DRAG_Y;
	rc.right = rc.left + 256 - 16;
	rc.bottom = rc.top + 256 - 16;
	ClipCursor(&rc);
}
void DirctDragPut(void)
{
	int  x, y, pos, piece;

	x = MOUSE_X / 64;
	y = MOUSE_Y / 64;
	pos = y * 4 + x;

	piece = PICBOX[pos];
	PICBOX[MovePiecePos] = piece;
	PICBOX[pos] = MovePieceNum;
	MovePieceNum = 0;
	PictureShow();

	ClipCursor(NULL);
}
void DirctDragMove(int mouse_on)
{
	if (mouse_on == FALSE) {
		PICBOX[MovePiecePos] = MovePieceNum;
		MovePieceNum = 0;
	}
	PictureShow();
}
//*****************************************************************************
// obN[h
//*****************************************************************************
void BackStatus(void)
{
	char buff[30];

	wsprintf(buff, " Backward %d moves", BackCount);
	SetWindowText(GetDlgItem(hDlog, IDC_STATUS1), buff);
}
void Backward(void)
{
	int  x, y, n, pos, pos1, pos2;

	if (BackCount == 999) return;

	x = MOUSE_X / 64;
	y = MOUSE_Y / 64;
	pos1 = y * 4 + x;
	n = PICBOX[pos1];
	if (n == 0) return;

	pos2 = -1;
	if (y > 0) {	// 
		pos = (y - 1) * 4 + x;
		if (PICBOX[pos] == 0) pos2 = pos;
	}
	if (y < 3) {	// 
		pos = (y + 1) * 4 + x;
		if (PICBOX[pos] == 0) pos2 = pos;
	}
	if (x > 0) {	// 
		pos = y * 4 + (x - 1);
		if (PICBOX[pos] == 0) pos2 = pos;
	}
	if (x < 3) {	// E
		pos = y * 4 + (x + 1);
		if (PICBOX[pos] == 0) pos2 = pos;
	}
	if (pos2 == -1) return;

	BackCount++;
	BackStatus();

	PICBOX[pos1] = 0;
	PICBOX[pos2] = n;
	PictureShow();
}
//*****************************************************************************
// Vbt
//*****************************************************************************
void Shuffle(void)
{
	int pos1, pos2, temp;

	do {
		// Vbt (PICBOX[15]->PICBOX[1] lɃ_)
		for (pos1 = BOARD_SIZE - 1; pos1; pos1--) {
			pos2 = (pos1 + 1) * rand() / (RAND_MAX + 1);
			temp = PICBOX[pos1]; PICBOX[pos1] = PICBOX[pos2]; PICBOX[pos2] = temp;
		}
	} while (!ParityCheck(PICBOX));	// ݂邩?

	PictureShow();
}
//*****************************************************************************
// EBhȄI
//*****************************************************************************
void QuitWindow(HWND hWnd)
{
	// PDB[
	free(PDB1);
	free(PDB2);

	DeleteDC(ItemDC);
	DeleteDC(BuffDC);
	DeleteObject(BuffBM);
}
//*****************************************************************************
// EBhȄ
//*****************************************************************************
void InitWindow(HWND hWnd)
{
	HBRUSH hBrush;
	RECT rcw, rc1, rc2;
	int  flame, title;

	// Av\
	SetWindowText(hWnd, szTitle);

	// 摜foCXReLXgƗʍ쐬
	HDC hdc = GetDC(hWnd);
	ItemDC = CreateCompatibleDC(hdc);
	BuffDC = CreateCompatibleDC(hdc);
	BuffBM = CreateCompatibleBitmap(hdc, 256, 256);
	SelectObject(BuffDC, BuffBM);

	// rbg}bv摜[h(\[X)
	ItemBM = LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP1));
	SelectObject(ItemDC, ItemBM);

	// rbg}bv摜̋}XwiFɂ
	hBrush = CreateSolidBrush(GetSysColor(COLOR_3DFACE));
	SelectObject(ItemDC, hBrush);
	PatBlt(ItemDC, 0, 0, 64, 64, PATCOPY);

	// ʕ\eLXgŒ蕝tHgɂ
	hFont = CreateFont(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, "FixedSys");
	SendMessage(GetDlgItem(hWnd, IDC_EDIT1), WM_SETFONT, (WPARAM)hFont, MAKELPARAM(TRUE, 0));

	// Window̃t[ƃ^Cg
	GetWindowRect(hWnd, &rcw);
	GetClientRect(hWnd, &rc1);
	flame = (rcw.right - rcw.left - rc1.right) / 2;
	title = (rcw.bottom - rcw.top - rc1.bottom) - flame;

	// sN`[{bNẌ擾
	GetWindowRect(GetDlgItem(hWnd, IDC_EDIT1), &rc1);
	Picture.left = ((rcw.left + flame) + rc1.left) / 2 - 128 - (rcw.left + flame);
	GetWindowRect(GetDlgItem(hWnd, IDC_GROUP1), &rc1);
	GetWindowRect(GetDlgItem(hWnd, IDC_BAR), &rc2);
	Picture.top = (rc1.bottom + rc2.top) / 2 - 128 - (rcw.top + title);
	Picture.right = Picture.left + 256;
	Picture.bottom = Picture.top + 256;

	// }EXhbOWindowszuɑ΂N_
	DRAG_X = flame + Picture.left + 8;
	DRAG_Y = title + Picture.top + 8;

	// V[h
	srand((unsigned int)time(NULL));

	// 
	Reset();
	Enable(hWnd, NORMAL);
	SetWindowText(GetDlgItem(hDlog, IDC_STATUS1), "Solver = WD + PDB + OpenMP");

	// PDB[m
	PDB1 = (char*)malloc(PDB_SIZE);
	PDB2 = (char*)malloc(PDB_SIZE);

	// PDBe[uWDe[uʃXbhō쐬
	EnableWindow(GetDlgItem(hWnd, IDC_SOLVE), FALSE);
	MakeCombin();													// nCr쐬
	_beginthread(Pdb1Thread, 0, NULL);	// PDB1쐬
	_beginthread(Pdb2Thread, 0, NULL);	// PDB2쐬
	_beginthread(WdThread, 0, NULL);	// WD쐬
}
//*****************************************************************************
// }EXCxg
//*****************************************************************************
BOOL MouseProc(UINT message)
{
	// }EX|C^sN`[Wɕϊ
	MOUSE_X -= Picture.left;
	MOUSE_Y -= Picture.top;

	switch (message) {
	// }EXDOWN
	case WM_LBUTTONDOWN:
		switch (Status) {
		case BACK: Backward(); return TRUE;
		case DRAG: DirctDragGet(); return TRUE;
		}
		break;
	// }EXUP
	case WM_LBUTTONUP:
		if (Status == DRAG && MovePieceNum) {
			DirctDragPut();
			return TRUE;
		}
		break;
	// }EXMOVE
	case WM_MOUSEMOVE:
		if (Status == DRAG && MovePieceNum) {
			DirctDragMove(TRUE);
			return TRUE;
		}
		break;
	}
	return FALSE;
}
//*****************************************************************************
// EBhEvV[W Ԃl(:TRUE	:FALSE)
//*****************************************************************************
BOOL CALLBACK WindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	char buff[100];

	switch (msg) {
	// 
	case WM_INITDIALOG:
		// nhۑ
		hDlog = hWnd;
		// C_CAOɃACRt
		SendMessage(hWnd, WM_SETICON, ICON_SMALL,
			(LPARAM)LoadIcon(hInst, MAKEINTRESOURCE(IDI_ICON1)));
		// EBhȄ
		InitWindow(hWnd);
		return TRUE;
	// R}h
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_SHUFFLE:
			Enable(hWnd, NORMAL);
			Shuffle();
			SendMessage(GetDlgItem(hWnd, IDC_BACK), BM_SETCHECK, FALSE, 0);
			SendMessage(GetDlgItem(hWnd, IDC_DRAG), BM_SETCHECK, FALSE, 0);
			SetWindowText(GetDlgItem(hDlog, IDC_STATUS1), "Shuffle");
			return TRUE;
		case IDC_BACK:
			Enable(hWnd, BACK);
			Reset();
			BackCount = 0;
			SendMessage(GetDlgItem(hWnd, IDC_DRAG), BM_SETCHECK, FALSE, 0);
			BackStatus();
			return TRUE;
		case IDC_DRAG:
			Enable(hWnd, DRAG);
			SendMessage(GetDlgItem(hWnd, IDC_BACK), BM_SETCHECK, FALSE, 0);
			SetWindowText(GetDlgItem(hDlog, IDC_STATUS1), "DirctDrag");
			return TRUE;
		case IDC_RESET:
			if (Status == BACK) {
				Reset();
				BackCount = 0;
				BackStatus();
				return 0;
			}
			if (Status == NORMAL || Status == DRAG) {
				Reset();
				if (Status == NORMAL) SetWindowText(GetDlgItem(hDlog, IDC_STATUS1), "DirctDrag");
				return 0;
			}
			if (Status == SHOW) {
				if (TraceIndex) {
					for (int i = 0; i < BOARD_SIZE; i++)
						PICBOX[i] = BOARD[i];
					TraceIndex = 0;
					PictureShow();
					wsprintf(buff, " %2d / %2d moves", TraceIndex, MOVES);
					SetWindowText(GetDlgItem(hDlog, IDC_STATUS1), buff);
				}
				else {
					Reset();
					Enable(hWnd, NORMAL);
				}
			}
			return TRUE;
		case IDC_SOLVE:
			Enable(hWnd, SOLVE);
			SetWindowText(GetDlgItem(hDlog, IDC_EDIT1), "");
			SendMessage(GetDlgItem(hWnd, IDC_BACK), BM_SETCHECK, FALSE, 0);
			SendMessage(GetDlgItem(hWnd, IDC_DRAG), BM_SETCHECK, FALSE, 0);
			NOW_TIME = 0;
			SetTimer(hWnd, (UINT)ID_MYTIMER, (UINT)1000, NULL);
			_beginthread(SolverThread, 0, NULL);
			return TRUE;
		case IDC_CANCEL:
			if (Status == SOLVE) {	// SolveLZ
				MOVES = -100;
			}
			else {					// TraceLZ
				PlayStop = TRUE;
			}
			return TRUE;
		case IDC_TRACE:
			Enable(hWnd, PLAY);
			SetTimer(hWnd, (UINT)ID_MYTIMER, (UINT)10, NULL);
			return TRUE;
		}
		break;
	// }EXCxg
	case WM_LBUTTONDOWN:
	case WM_LBUTTONUP:
	case WM_MOUSEMOVE:
		if (Status != BACK && Status != DRAG) break;
		MOUSE_X = (int)LOWORD(lParam);
		MOUSE_Y = (int)HIWORD(lParam);
		if (MOUSE_X >= Picture.left && MOUSE_X <= Picture.right &&
			MOUSE_Y >= Picture.top && MOUSE_Y <= Picture.bottom)
			if (MouseProc(msg)) return 0;
		break;
	// 荞݃Cxg
	case UM_PDB1_END:	// PDBe[u쐬I
		PDB1_END = 1;
		if (PDB2_END && WD_END) EnableWindow(GetDlgItem(hWnd, IDC_SOLVE), TRUE);
		return 0;
	case UM_PDB2_END:	// PDBe[u쐬I
		PDB2_END = 1;
		if (PDB1_END && WD_END) EnableWindow(GetDlgItem(hWnd, IDC_SOLVE), TRUE);
		return 0;
	case UM_WD_END: 	// WDe[u쐬I
		WD_END = 1;
		if (PDB1_END && PDB2_END) EnableWindow(GetDlgItem(hWnd, IDC_SOLVE), TRUE);
		return 0;
	case UM_SOLVE_END:	// \o[I
		KillTimer(hWnd, (UINT)ID_MYTIMER);
		Enable(hWnd, SHOW);
		return 0;
	// ^C}[Cxg
	case WM_TIMER:
		if (Status == PLAY) {	// TraceAj[V
			if (TraceIndex < MOVES || MovePieceNum) {
				Animation(hWnd);
			}
			else {
				KillTimer(hWnd, (UINT)ID_MYTIMER);
				Sleep(1500);
				for (int i = 0; i < BOARD_SIZE; i++)
					PICBOX[i] = BOARD[i];
				TraceIndex = 0;
				MovePieceNum = 0;
				PictureShow();
				Enable(hWnd, SHOW);
				wsprintf(buff, " %2d / %2d moves", TraceIndex, MOVES);
				SetWindowText(GetDlgItem(hDlog, IDC_STATUS1), buff);
			}
		}
		else {	// Status = SOLVE (oߎԂ̕\)
			wsprintf(buff, "%5d sec", ++NOW_TIME);
			SetWindowText(GetDlgItem(hDlog, IDC_STATUS2), buff);
		}
		return 0;
	// ĕ`
	case WM_PAINT:
		PAINTSTRUCT ps;
		BeginPaint(hWnd, &ps);
		BitBlt(ps.hdc, Picture.left, Picture.top, 256, 256, BuffDC, 0, 0, SRCCOPY);
		EndPaint(hWnd, &ps);
		return TRUE;
	// I
	case WM_CLOSE:
		// EBhȄI
		QuitWindow(hWnd);
		// C_CAO
		EndDialog(hWnd, 0);
		return TRUE;
	}
	return FALSE;
}
//*****************************************************************************
// Gg[|Cg
//*****************************************************************************
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR lpCmdLine, int nCmdShow)
{
	// CX^Xۑ
	hInst = hInstance;

	// C_CAOJ
	DialogBox(hInstance, MAKEINTRESOURCE(IDD_DIALOG1), NULL, (DLGPROC)WindowProc);

	return 0;
}
